@@ -27,7 +27,10 @@ import cairo
 import urllib2
 import gtk
 import time
+import cairo
+import traceback
 import modrana_utils
+import rectangles
 from configobj import ConfigObj
 from tilenames import *
 
@@ -43,7 +46,7 @@ socket.setdefaulttimeout(timeout)
 #  return
 
 def getModule(m,d,i):
-  return(mapTiles(m,d,i))
+  return(MapTiles(m,d,i))
 
 maplayers = {}
 configVariables = {
@@ -87,12 +90,12 @@ except Exception, e:
 
 
   
-class mapTiles(ranaModule):
+class MapTiles(ranaModule):
   """Display map images"""
   def __init__(self, m, d, i):
     ranaModule.__init__(self, m, d, i)
     self.images = [{},{}] # the first dict contains normal image data, the seccond contains special tiles
-    self.imagesLock = threading.Lock()
+    self.imagesLock = threading.RLock()
     self.threads = {}
     self.threadlListCondition = threading.Condition(threading.Lock())
     self.maxImagesInMemmory = 150 # to avoid a memmory leak
@@ -134,7 +137,32 @@ class mapTiles(ranaModule):
 
   def firstTime(self):
     self.mapViewModule = self.m.get('mapView', None)
+    self._updateScalingCB()
+    self.modrana.watch('mapScale', self._updateScalingCB)
+    self.modrana.watch('z', self._updateScalingCB)
+
+  def _updateScalingCB(self, key=None, oldValue=None, newValue=None):
+    """
+    as this only needs to be updated once on startup and then only
+    when scaling settings change this callback driven method is used
+    """
+
+    scale = int(self.get('mapScale', 1))
+    if scale == 1: # this will be most of the time, so it is first
+      z = int(self.get('z', 15))
+    elif scale == 2: # tiles are scaled to 512*512 and represent tiles from zl-1
+      z = int(self.get('z', 15)) - 1
+    elif scale == 4: # tiles are scaled to 1024*1024 and represent tiles from zl-2
+      z = int(self.get('z', 15)) - 2
+    else:
+      z = int(self.get('z', 15))
+      
+    tileSide = self.tileSide * scale
     
+    self.scalingInfo = (scale, z, tileSide)
+
+
+
   def startTileDownloadManagementThread(self):
     """start the consumer thread for download requests"""
     t = Thread(target=self.tileDownloadManager, name='automatic tile download management thread')
@@ -165,7 +193,7 @@ class mapTiles(ranaModule):
                   request = self.downloadRequestPool.pop() # download most recent requests first
                   (name,x,y,z,layer,layerPrefix,layerType, filename, folder, timestamp) = request
                   # start a new thread
-                  self.threads[name] = self.tileDownloader(name,x,y,z,layer,layerPrefix,layerType, filename, folder, self)
+                  self.threads[name] = self.TileDownloader(name,x,y,z,layer,layerPrefix,layerType, filename, folder, self)
                   self.threads[name].daemon = True
                   self.threads[name].start()
                   # chenge the status tile to "Downloading..."
@@ -279,41 +307,45 @@ class mapTiles(ranaModule):
   def drawMap(self, cr):
     """Draw map tile images"""
     try: # this should get rid of a fair share of the infamous "black screens"
+      # get all neded data and objects to local variables
       proj = self.m.get('projection', None)
+      drawImage = self.drawImage # method binding to speed up method lookup
+      overlay = self.get('overlay', False)
+      """ if overlay is enabled, we use a special naming
+          function in place of the default one"""
+
+      singleGetName = self.getTileName
+      if overlay:
+        ratio = self.get('transpRatio', "0.5,1").split(',') # get the transparency ratio
+        (alphaOver, alphaBack) = (float(ratio[0]),float(ratio[1])) # convert it to floats
+        layer1 = self.get('layer','osma')
+        layer2 = self.get('layer2', 'mapnik')
+        layerInfo = ((layer1, alphaBack),(layer2, alphaOver))
+        getName = self.getCompositeTileName
+      else:
+        layerInfo = self.get('layer','osma')
+        getName = self.getTileName
+
       if proj and proj.isValid():
         loadingTileImageSurface = self.loadingTile[0]
         requests = []
         (sx,sy,sw,sh) = self.get('viewport') # get screen parameters
-        scale = int(self.get('mapScale', 1)) # get the current scale
+
+        # adjust left corner ccordinates if cewntering shift is on
+        (shiftX,shiftY) = self.modrana.centerShift
+        sx = -shiftX
+        sy = -shiftY
+        # get the current scale and related info
+        (scale, z, tileSide) = self.scalingInfo
 
         if scale == 1: # this will be most of the time, so it is first
-          z = int(self.get('z', 15))
           (px1,px2,py1,py2) = (proj.px1,proj.px2,proj.py1,proj.py2) #use normal projection bbox
-          cleanProjectionCoords = (px1,px2,py1,py2) # wee need the unmodified coords for later use
+          cleanProjectionCoords = (px1,px2,py1,py2) # we need the unmodified coords for later use
         else:
-          if scale == 2: # tiles are scaled to 512*512 and represent tiles from zl-1
-            z = int(self.get('z', 15)) - 1
-          elif scale == 4: # tiles are scaled to 1024*1024 and represent tiles from zl-2
-            z = int(self.get('z', 15)) - 2
-          else:
-            z = int(self.get('z', 15))
-
-          # we use tiles from an upper zl and strech them over lower zl
+          # we use tiles from an upper zl and strech them over a lower zl
           (px1,px2,py1,py2) = proj.findEdgesForZl(z, scale)
           cleanProjectionCoords = (px1,px2,py1,py2) # wee need the unmodified coords for later use
 
-        if self.get("rotateMap", False) and (self.get("centred", False)):
-          # due to the rotation, the map must be larger
-          # we take the longest side and render tiles in a square
-          longestSide = max(sw,sh)
-          add = (longestSide/2.0)/(self.tileSide)
-          # enlarge the bounding box
-          (px1,px2,py1,py2) = (px1-add,px2+add,py1-add,py2+add)
-
-        # get the range of tiles we need
-        wTiles =  len(range(int(floor(px1)), int(ceil(px2)))) # how many tiles wide
-        hTiles =  len(range(int(floor(py1)), int(ceil(py2)))) # how many tiles high
-
         # upper left tile
         cx = int(px1)
         cy = int(py1)
@@ -324,104 +356,201 @@ class mapTiles(ranaModule):
         cx1,cy1 = (sw*(cx-px1)/pdx,sh*(cy-py1)/pdy) #this is basically the pxpy2xy function from mod_projection inlined
         cx1,cy1 = int(cx1),int(cy1)
 
-        layer = self.get('layer','osma')
-        # Cover the whole map view with tiles
+        if self.get("rotateMap", False) and (self.get("centred", False)):
+          # due to the rotation, the map must be larger
+          # we take the longest side and render tiles in a square
 
-        if self.get('overlay', False): # is the overlay on ?
-          ratio = self.get('transpRatio', "0.5,1").split(',') # get the transparency ratio
-          (alphaOver, alphaBack) = (float(ratio[0]),float(ratio[1])) # convert it to floats
+          # get the rotation angle
+          angle = self.get('bearing', 0.0)
+          if angle == None:
+            angle = 0.0
 
-          # get the background layer
-          layer2 = self.get('layer2', 'mapnik')
+          radAngle = radians(angle)
 
-          # draw the composited layer
-          with self.imagesLock: # just one lock per frame
+
+          # get the rotation angle from the main class (for synchronization purposses)
+          radAngle = radians(self.modrana.mapRotationAngle)
+
+          # we use polygon overlap testing to only load@draw visible tiles
+
+          # screen center point
+          (shiftX,shiftY) = self.modrana.centerShift
+          (centerX,centerY) = ((sw/2.0),(sh/2.0))
+          scP = rectangles.Point(centerX,centerY)
+
+          """create a polygon representing the viewport and rotate around
+          current rotation center it to match the rotation and align with screen"""
+          p1 = rectangles.Point(sx,sy)
+          p2 = rectangles.Point(sx+sw,sy)
+          p3 = rectangles.Point(sx,sy+sh)
+          p4 = rectangles.Point(sx+sw,sy+sh)
+          p1 = p1.rotate_about(scP,radAngle)
+          p2 = p2.rotate_about(scP,radAngle)
+          p3 = p3.rotate_about(scP,radAngle)
+          p4 = p4.rotate_about(scP,radAngle)
+
+          v1 = rectangles.Vector(*p1.as_tuple())
+          v2 = rectangles.Vector(*p2.as_tuple())
+          v3 = rectangles.Vector(*p3.as_tuple())
+          v4 = rectangles.Vector(*p4.as_tuple())
+          polygon = rectangles.Polygon((v1,v2,v3,v4))
+
+          v1 = rectangles.Vector(*p1.as_tuple())
+          v2 = rectangles.Vector(*p2.as_tuple())
+          v3 = rectangles.Vector(*p3.as_tuple())
+          v4 = rectangles.Vector(*p4.as_tuple())
+
+          # enlage the area of possibly visible tiles due to rotation
+          add = self.modrana.expandViewportTiles
+          (px1,px2,py1,py2) = (px1-add,px2+add,py1-add,py2+add)
+          cx = int(px1)
+          cy = int(py1)
+          (pdx, pdy) = (px2 - px1,py2 - py1)
+          cx1,cy1 = (cx1-add*tileSide,cy1-add*tileSide)
+
+          wTiles =  len(range(int(floor(px1)), int(ceil(px2)))) # how many tiles wide
+          hTiles =  len(range(int(floor(py1)), int(ceil(py2)))) # how many tiles high
+
+          visibleCounter = 0
+          with self.imagesLock:
             for ix in range(0, wTiles):
-              for iy in range(0, hTiles):
+                for iy in range(0, hTiles):
+                  tx = cx+ix
+                  ty = cy+iy
+                  stx = cx1 + tileSide*ix
+                  sty = cy1 + tileSide*iy
+                  tv1 = rectangles.Vector(stx,sty)
+                  tv2 = rectangles.Vector(stx+tileSide,sty)
+                  tv3 = rectangles.Vector(stx,sty+tileSide)
+                  tv4 = rectangles.Vector(stx+tileSide,sty+tileSide)
+                  tempPolygon = rectangles.Polygon((tv1,tv2,tv3,tv4))
+                  if polygon.intersects(tempPolygon):
+                    visibleCounter+=1
+                    (x,y,x1,y1) = (tx,ty,cx1 + tileSide*ix,cy1 + tileSide*iy)                  
+                    name = getName(layerInfo,z,x,y)
+                    tileImage = self.images[0].get(name)
+                    if tileImage:
+                      # tile found in memmory cache, draw it
+                      drawImage(cr, tileImage[0], x1, y1, scale)
+                    else:
+                      if overlay:
+                        """ check if the separate tiles are already cached
+                        and send loading request/-s if not
+                        if both tiles are in the cache, combine them, cache and disaplay the result
+                        and remove the separate tiles from cache
+                        """
+                        layerBack = layerInfo[0][0]
+                        layerOver = layerInfo[1][0]
+                        nameBack = singleGetName(layer1,z,x,y)
+                        nameOver = singleGetName(layer2,z,x,y)
+                        backImage = self.images[0].get(nameBack)
+                        overImage = self.images[0].get(nameOver)
+                        if backImage and overImage: # both images available
+                          if backImage[1]['type'] == "normal" and overImage[1]['type'] == "normal":
+                            combinedImage = self.combine2Tiles(backImage[0], overImage[0], alphaOver)
+                            # remove the separate images from cache
+                            self.removeImageFromMemmory(nameBack)
+                            self.removeImageFromMemmory(nameOver)
+                            # cache the combined image
+                            self.storeInMemmory(combinedImage, name, "composite")
+                            # draw the composite image
+                            drawImage(cr, combinedImage, x1, y1, scale)
+                          else:
+                            drawImage(cr, loadingTileImageSurface, x1, y1, scale)
+                        else:
+                          requests.append((nameBack, x, y, z, layerBack))
+                          requests.append((nameOver, x, y, z, layerOver))
+                          drawImage(cr, loadingTileImageSurface, x1, y1, scale)
+                      else:
+                        # tile not found in memmory cache, add a loading request
+                        requests.append((name, x, y, z, layerInfo))
+            if self.modrana.showRedrawTime:
+              print "currently visible tiles: %d/%d" % (visibleCounter,wTiles*hTiles)
 
-                # get tile cooridnates by incrementing the upper left tile cooridnates
-                x = cx+ix
-                y = cy+iy
+#            cr.set_source_rgba(0,1,0,0.5)
+#            cr.move_to(*p1.as_tuple())
+#            cr.line_to(*p2.as_tuple())
+#            cr.line_to(*p4.as_tuple())
+#            cr.line_to(*p3.as_tuple())
+#            cr.line_to(*p1.as_tuple())
+#            cr.close_path()
+#            cr.fill()
+#
+#            cr.set_source_rgba(1,0,0,1)
+#            cr.rectangle(scP.x-10,scP.y-10,20,20)
+#            cr.fill()
 
-                # get screen coordinates by incrementing upper left tile screen coordinates
-                x1 = cx1 + 256*ix*scale
-                y1 = cy1 + 256*iy*scale
+        else:
+          # draw without rotation
+          wTiles =  len(range(int(floor(px1)), int(ceil(px2)))) # how many tiles wide
+          hTiles =  len(range(int(floor(py1)), int(ceil(py2)))) # how many tiles high
 
-                # Try to load and display images
-                nameBack = "%s_%d_%d_%d" % (layer2,z,x,y)
-                nameOver = "%s_%d_%d_%d" % (layer,z,x,y)
-                backImage = self.images[0].get(nameBack)
-                overImage = self.images[0].get(nameOver)
-                # check if the background tile is already cached
-                if backImage == None: # background image not yet loaded
-                  requests.append((nameBack, x, y, z, layer2))
-                  backImage = [loadingTileImageSurface] # draw the loading tile this time
-
-                # check if the overlay tile is already cached
-                if overImage == None: # overlay image not yet loaded
-                  requests.append((nameOver, x, y, z, layer))
-                  overImage = [loadingTileImageSurface] # draw the loading tile this time
-
-                """we do this inline to get rid of function calling overhead"""
-                # Move the cairo projection onto the area where we want to draw the image
-                cr.save()
-                cr.translate(x1,y1)
-                cr.scale(scale,scale) # scale te tile according to current scale settings
-
-                # Display the image
-                cr.set_source_surface(backImage[0],0,0) # draw the background
-                cr.paint_with_alpha(alphaBack)
-                cr.set_source_surface(overImage[0],0,0) # draw the overlay
-                cr.paint_with_alpha(alphaOver)
-
-                # Return the cairo projection to what it was
-                cr.restore()
-
-        else: # overlay is disabled
           with self.imagesLock: # just one lock per frame
             # draw the normal layer
             for ix in range(0, wTiles):
               for iy in range(0, hTiles):
-
                 # get tile cooridnates by incrementing the upper left tile cooridnates
                 x = cx+ix
                 y = cy+iy
 
                 # get screen coordinates by incrementing upper left tile screen coordinates
-                x1 = cx1 + 256*ix*scale
-                y1 = cy1 + 256*iy*scale
+                x1 = cx1 + tileSide*ix
+                y1 = cy1 + tileSide*iy
 
                 # Try to load and display images
-                """we do this inline to get rid of function calling overhead"""
-                name = "%s_%d_%d_%d" % (layer,z,x,y)
+                name = getName(layerInfo,z,x,y)
                 tileImage = self.images[0].get(name)
                 if tileImage:
                   # tile found in memmory cache, draw it
-                  cr.save() # save the cairo projection context
-                  cr.translate(x1,y1)
-                  cr.scale(scale,scale)
-                  cr.set_source_surface(tileImage[0],0,0)
-                  cr.paint()
-                  cr.restore() # Return the cairo projection to what it was
-                else:       
-                  # tile not found in memmory cache, add a loading request
-                  requests.append((name, x, y, z, layer))
-                  cr.save() # save the cairo projection context
-                  cr.translate(x1,y1)
-                  cr.scale(scale,scale)
-                  cr.set_source_surface(loadingTileImageSurface,0,0)
-                  cr.paint()
-                  cr.restore() # Return the cairo projection to what it was
-        # batch send the requests to the loading thread
-        if requests:
-          self.loadRequestCStack.batchPush(requests)
-          """can the loadRequestCStack get full ?
-             NO :)
-             it takes the list, reverses it, extends its old internal
-             list with it - from this list a slice conforming to
-             its size limit is taken and set as the new internal list"""
-            # notify the loading thread using the notify Queue
-          self.loadingNotifyQueue.put("load", block=False)
+                  drawImage(cr, tileImage[0], x1, y1, scale)
+                else:
+                  # tile not found im memmroy cache, do something else
+                  if overlay:
+                    """ check if the separate tiles are already cached
+                    and send loading request/-s if not
+                    if both tiles are in the cache, combine them, cache and disaplay the result
+                    and remove the separate tiles from cache
+                    """
+                    layerBack = layerInfo[0][0]
+                    layerOver = layerInfo[1][0]
+                    nameBack = singleGetName(layer1,z,x,y)
+                    nameOver = singleGetName(layer2,z,x,y)
+                    backImage = self.images[0].get(nameBack)
+                    overImage = self.images[0].get(nameOver)
+                    if backImage and overImage: # both images available
+                      if backImage[1]['type'] == "normal" and overImage[1]['type'] == "normal":
+                        """
+                        we check the the metadata to filter out the "Downloading..."
+                        special tiles
+                        """
+                        combinedImage = self.combine2Tiles(backImage[0], overImage[0], alphaOver)
+                        # remove the separate images from cache
+                        self.removeImageFromMemmory(nameBack)
+                        self.removeImageFromMemmory(nameOver)
+                        # cache the combined image
+                        self.storeInMemmory(combinedImage, name, "composite")
+                        # draw the composite image
+                        drawImage(cr, combinedImage, x1, y1, scale)
+                      else: # on or more tiles not usable
+                        drawImage(cr, loadingTileImageSurface, x1, y1, scale)
+                    else:
+                      requests.append((nameBack, x, y, z, layerBack))
+                      requests.append((nameOver, x, y, z, layerOver))
+                      drawImage(cr, loadingTileImageSurface, x1, y1, scale)
+                  else:
+                    # tile not found in memmory cache, add a loading request
+                    requests.append((name, x, y, z, layerInfo))
+                    drawImage(cr, loadingTileImageSurface, x1, y1, scale)
+
+      if requests:
+        self.loadRequestCStack.batchPush(requests)
+        """can the loadRequestCStack get full ?
+           NO :)
+           it takes the list, reverses it, extends its old internal
+           list with it - from this list a slice conforming to
+           its size limit is taken and set as the new internal list"""
+          # notify the loading thread using the notify Queue
+        self.loadingNotifyQueue.put("load", block=False)
 
     except Queue.Full:
       """as we use the queue as a notification mechanism, we dont actually need to
@@ -430,25 +559,38 @@ class mapTiles(ranaModule):
 
     except Exception, e:
       print "mapTiles: expception while drawing the map layer: %s" % e
+      traceback.print_exc()
 
+  def drawImage(self, cr, imageSurface, x, y, scale):
+    """draw a map tile image"""
+    cr.save() # save the cairo projection context
+    cr.translate(x,y)
+    cr.scale(scale,scale)
+    cr.set_source_surface(imageSurface,0,0)
+    cr.paint()
+    cr.restore() # Return the cairo projection to what it was
 
+  def getTileName(self, layer, z, x, y):
+    return "%s_%d_%d_%d" % (layer,z,x,y)
+
+  def getCompositeTileName (self, layers , z, x, y):
+    (layer1, layer2) = layers
+    return "%s-%d+%s-%d:%d_%d_%d" % (layer1[0], layer1[1], layer2[0], layer2[1] , z, x, y)
+
+  def combine2Tiles(self, backImage, overImage, alphaOver):
+    # transparently combine two tiles
+    ct = cairo.Context(backImage)
+    ct2 = gtk.gdk.CairoContext(ct)
+    ct2.set_source_surface(overImage,0,0)
+    ct2.paint_with_alpha(alphaOver)
+    return backImage
+  
   def removeImageFromMemmory(self, name, dictIndex=0):
     # remove an image from memmory
     with self.imagesLock: #make sure no one fiddles with the cache while we are working with it
       if name in self.images:
         del self.images[dictIndex][name]
   
-  def drawImage(self,cr, tileImage, x, y, scale):
-    """Draw a tile image"""
-    # move to the drawing coordinates
-    cr.translate(x1,y1)
-    cr.scale(scale,scale) # scale te tile accorind to current scale settings
-    
-    # Display the image
-    cr.set_source_surface(tileImage,0,0)
-    # paint the result
-    cr.paint()
-
   def drawCompositeImage(self,cr, nameOver, nameBack, x,y, scale, alpha1=1, alpha2=1, dictIndex1=0,dictIndex2=0):
     """Draw a composited tile image"""
 
@@ -478,6 +620,8 @@ class mapTiles(ranaModule):
           with self.imagesLock: # substitute the "loading" tile with a "downloading" tile
             downloadingTile = self.downloadingTile
             downloadingTile[1]['addedTimestamp'] = time.time()
+            downloadingTile[1]['type'] = "downloading"
+
             self.images[0][name] = downloadingTile
           return('OK')
     
@@ -499,31 +643,6 @@ class mapTiles(ranaModule):
         return('OK')
 
     filename = self.getTileFolderPath() + (self.getImagePath(x,y,z,layerPrefix, layerType))
-#
-#    storageType = self.get('tileStorageType', 'files')
-#    if storageType == 'sqlite': # use the sqlite based storage method
-#      m = self.m.get('storeTiles', None) # get the tile storage module
-#      if m:
-#        try:
-#          buffer = m.getTile(layerPrefix, z, x, y, layerType)
-#          if buffer:
-#            pl = gtk.gdk.PixbufLoader()
-#            pl.write(buffer)
-#            pl.close()
-#            pixbuf = pl.get_pixbuf()
-#            self.storeInMemmory(self.pixbufToCairoImageSurface(pixbuf), name)
-#            return('OK')
-#        except Exception, e:
-#          print "loading tile from sqlite failed"
-#          print "exception: ", e
-#    else: #use the default method -> load from files
-#      if (os.path.exists(filename)):
-#        try:
-#          pixbuf = gtk.gdk.pixbuf_new_from_file(filename)
-#          self.storeInMemmory(self.pixbufToCairoImageSurface(pixbuf), name)
-#        except Exception, e:
-#          print "the tile image is corrupted nad/or there are no tiles for this zoomlevel, exception:\n%s" % e
-#        return('OK')
 
     # Image not found anywhere locally - resort to downloading it
 
@@ -658,7 +777,7 @@ class mapTiles(ranaModule):
     """helper function that returns path to the tile folder"""
     return self.get('tileFolder', 'cache/images')
 
-  def imageY(z,extension):
+  def imageY(self, z,extension):
     return (('%d.%s') % (z, extension))
 
   def layers(self):
@@ -682,7 +801,7 @@ class mapTiles(ranaModule):
     with self.threadlListCondition:
       self.threadlListCondition.notifyAll()
 
-  class tileDownloader(Thread):
+  class TileDownloader(Thread):
     """Downloads an image (in a thread)"""
     def __init__(self,name, x,y,z,layer,layerName,layerType,filename, folder, callback):
       Thread.__init__(self)
@@ -811,7 +930,7 @@ def getTileUrl(x,y,z,layer): #TODO: share this with mapData
         layerDetails['tiles'],
         x,y,z)
     elif coords == 'quadtree': # handle Virtual Earth maps and satelite
-      quadKey = QuadTree(x, y, z)
+      quadKey = quadTree(x, y, z)
       url = '%s%s?g=452' % ( #  dont know what the g argument is, maybe revision ? but its not optional
                                 layerDetails['tiles'], # get the url
                                 quadKey # get the tile identificator
@@ -832,7 +951,7 @@ def getTileUrl(x,y,z,layer): #TODO: share this with mapData
 
   
 # modified from: http://www.maptiler.org/google-maps-coordinates-tile-bounds-projection/globalmaptiles.py (GPL)
-def QuadTree(tx, ty, zoom ):
+def quadTree(tx, ty, zoom ):
 		"Converts OSM type tile coordinates to Microsoft QuadTree"
 
 		quadKey = ""
