@@ -32,27 +32,35 @@ class options(ranaModule):
     self.load()
     self.on = '<span color="green">ON</span>'
     self.off = '<span color="red">OFF</span>'
-  
-  def addCategory(self,name,id,icon,actionPrefix="",actionSufix=""):
+
+  def _getCategoryID(self, id):
+    return "opt_cat_%s" % id # get a standardized id
+
+  def addCategory(self,name,inId,icon,actionPrefix="",actionSufix=""):
     """this metohod shuld be run only after menu module instance
     is vailable in self.menuModule and the options manu is clared,
     eq. has at least the escape button"""
     # firs we add the category button to the options
-    id = "opt_cat_%s" % id # get a standardized id
+    id = self._getCategoryID(inId) # get a standardized id
     action="%sset:menu:%s%s" % (actionPrefix,id,actionSufix)
     self.menuModule.addItem('options', name, icon, action)
     # intilize menu for the new category menu
     self.menuModule.clearMenu(id,"set:menu:options")
     # as a convenience, return the id
-    return id
+    return inId
 
+  def _getGroupId(self, catId, id):
+    parrentId = self._getCategoryID(catId)
+    return "%s_opt_group_%s" % (parrentId, id)
+  
   def addGroup(self,name,id,parrentId,icon,actionPrefix="",actionSufix=""):
     """this method ads a new (empty) options group to category specified by
     catId, as a convenience feature, the id of the new group is returned"""
-    id = "opt_group_%s_%s" % (id,parrentId) # get a standardized id
+    catId = self._getCategoryID(parrentId)
+    id = self._getGroupId(parrentId, id) # get a standardized id
     action="%sset:menu:%s%s" % (actionPrefix,id,actionSufix)
-    self.menuModule.addItem(parrentId, name, icon, action)
-    self.options[id] = ["set:menu:%s" % parrentId,0,[]]
+    self.menuModule.addItem(catId, name, icon, action)
+    self.options[id] = ["set:menu:%s" % catId,0,[]]
     return id
 
   def addBoolOption(self, title, variable, group, default=None, action=None):
@@ -63,12 +71,33 @@ class options(ranaModule):
     else:
       self.addOption(title,variable,((False,off),(True,on)),group,default)
 
+  def addEditOption(self, title, variable, group, label="Edit variable", description=None):
+    choices = {"type":"showAndEditVariable",
+               "label": label,
+               "description": description
+              }
+    self.addOption(title, variable, choices, group, None)
+
   def addOption(self, title, variable, choices, group, default=None):
+
     newOption = (title,variable, choices,group,default)
     if self.options.has_key(group):
       self.options[group][2].append(newOption)
     else:
       print "options: group %s does not exist, call addGroup to create it first" % group
+
+  def removeOption(self, categoryId, groupId, variable):
+    """remova an option given by group and variable name"""
+
+    group = self._getGroupId(categoryId, groupId)
+
+    if self.options.has_key(group):
+      remove = lambda x:x[1]==variable
+      self.options[group][2][:] = [x for x in self.options[group][2] if not remove(x)]
+    else:
+      print "options: group %s does not exist, so option with variable %s can not be removed" % (group,variable)
+
+#  def optionExists(self, categoryId, groupId):
     
   def firstTime(self):
     """Create a load of options.  You can add your own options in here,
@@ -117,10 +146,7 @@ class options(ranaModule):
 
       # ** Rotation
       group = addGroup("Rotation", "map_rotation", catMap, "generic")
-      addBoolOpt("Rotate map in direction of travel", "rotateMap", group, False, 'device:updateAppMenu')
-      """the device module might show a device specific app-menu, so we have to notify it
-      about a chnage in the map rotation state"""
-
+      addBoolOpt("Rotate map in direction of travel", "rotateMap", group, False)
 
       # ** Scaling
       group = addGroup("Scaling", "map_scaling", catMap, "generic")
@@ -280,7 +306,7 @@ class options(ranaModule):
       # * the Keys category *
       catKeys = addCat("Keys", "keys", "keys")
       # * Device buttons
-      group = addGroup("Device buttons", "device_buttons", catKeys, "generic")
+      group = addGroup("Device buttons", "device_buttons", catKeys, "n900")
       if self.dmod.hasVolumeKeys():
         addBoolOpt("Use volume keys for zooming", "useVolumeKeys", group, True, "device:updateKeys")
 
@@ -320,9 +346,10 @@ class options(ranaModule):
                           ('tr tr', 'Turkish'),
                           ('vi vi', 'Vietnamese')]
 
-    addOpt("Language for directions", group, directionsLanguages,
-       group,
-       "en en") # TODO: use locale for default language ?
+    addOpt("Language for directions", 'directionsLanguage',
+         directionsLanguages,
+         group,
+         "en en") # TODO: use locale for default language ?
 
     addOpt("Autostart navigation", "autostartNavigationDefaultOnAutoselectTurn",
       [('disabled',"OFF"),
@@ -439,9 +466,23 @@ class options(ranaModule):
     # * the Sound category *
     catSound = addCat("Sound", "sound", "sound")
     # * sound output
-    group = addGroup("Sound output", "sound_output", catSound, "generic")
+    group = addGroup("Sound output", "sound_output", catSound, "sound")
 
-    addBoolOpt("Application wide sound output", "soundEnabled", group, True, "device:updateAppMenu")
+    addBoolOpt("Application wide sound output", "soundEnabled", group, True)
+
+    # * espeak group
+    group = addGroup("Voice", "voice_out", catSound, "espeak")
+
+    addOpt("Voice parameters","voiceParameters",
+      [("auto", "<b>automatic</b>","ms:options:espeakParams:auto"),
+       ("manual", "<b>manual</b>", "ms:options:espeakParams:manual")],
+       group,
+       "auto")
+
+    if self.get('voiceParameters', None) == "manual":
+      self._updateVoiceManual('add')
+
+       
 
 #    addOpt("Network", "threadedDownload",
 ##      [("off","No use of network"),
@@ -498,8 +539,9 @@ class options(ranaModule):
 #     ("simple", "Draw simple tracklogs")],
 #     "view",
 #     False)
+    self._setUndefinedToDefault()
 
-
+  def _setUndefinedToDefault(self):
     # Set all undefined options to default values
     for category,options in self.options.items():
       for option in options[2]:
@@ -562,10 +604,49 @@ class options(ranaModule):
         self.options[menuName][1] = newIndex
     elif(message == "save"):
       self.save()
-    
+    elif type=="ms" and message == "espeakParams":
+      # switch between espeak parameter modes
+      if args == "manual":
+        self._updateVoiceManual("remove")
+      elif args == "auto":
+        self._updateVoiceManual("add")
+
+    elif type == "ml" and message == "editVariable":
+      (variable, label, description) = args
+      initialText = self.get(variable, "")
+      entry = self.m.get('textEntry', None)
+      if entry:
+        key = "editVariable_%s" % variable
+        entry.entryBox(self,key, label, initialText, description)
+
+  def _updateVoiceManual(self, action):
+    """add or remove custom voce parameters option items"""
+
+    if action == "add":
+      groupId = self._getGroupId("sound", "voice_out")
+      description="<b>Note:</b> <tt>%language%</tt> will be replaced by current language code, <tt>%message%</tt> will be replaced by the message and <tt>%qmessage%</tt> will be replaced by the message in quotes"
+
+      self.addEditOption("Edit voice string", "voiceString", groupId, "Edit voice string", description=description)
+
+      message = "ms:voice:resetStringToDefault:espeak"
+      self.addOption("Reset voice string with <b>Espeak</b> default", "placeholder",
+      [("foo","<i>click to use this default</i>",message)],
+       groupId,
+       "foo")
+
+    elif action == "remove":
+      self.removeOption("sound", "voice_out", "voiceString")
+      self.removeOption("sound", "voice_out", "placeholder")
+
+  def handleTextEntryResult(self, key, result):
+    (type, variable) = key.split("_", 1)
+    if type == "editVariable":
+      print "editing variable: %s with: %s" % (variable, result)
+      self.set(variable,result)
+
   def drawMenu(self, cr, menuName):
     """Draw menus"""
-    if(menuName[0:5] != "opt_g"):
+    if(menuName[0:5] != "opt_c"):
       return
     if(not self.options.has_key(menuName)):
       return
@@ -599,6 +680,7 @@ class options(ranaModule):
       for row in (0,1,2):
         index = firstItemIndex + row
         numItems = len(options)
+        cAction = None
         if(0 <= index < numItems):
           (title,variable,choices,category,default) = options[index]
           # What's it set to currently?
@@ -608,29 +690,41 @@ class options(ranaModule):
           # (if any, use str(value) if it doesn't match any defined options)
           # Also lookup the _next_ choice in the list, because that's what
           # we will set the option to if it's clicked
-          nextChoice = choices[0]
-          valueDescription = str(value)
-          useNext = False
-          for c in choices:
-            cAction = None
-            if len(c) == 3:
-              (cVal, cName, cAction) = c
-            else:
-              (cVal, cName) = c
-            if(useNext):
-              nextChoice = c
-              useNext = False
-            if(str(value) == str(cVal)):
-              valueDescription = cName
-              useNext = True
 
-          # What should happen if this option is clicked -
-          # set the associated option to the next value in sequence
-          onClick = "set:%s:%s" % (variable, str(nextChoice[0]))
-          if cAction:
-            onClick += "|%s" % cAction
-          onClick += "|options:save"
-          onClick += "|set:needRedraw:1"
+          if type(choices) is dict:
+            optionType = choices["type"]
+            label = choices["label"]
+            description = choices["description"]
+
+            if optionType == "showAndEditVariable":
+              valueDescription = self.get(variable, "variable is not set yet")
+              valueDescription = "<tt><b>%s</b></tt>" % valueDescription
+              payload = "%s;%s;%s" % (variable, label, description)
+              onClick = "ml:options:editVariable:%s|set:needRedraw:True" % payload
+
+          else: # toggle button
+
+            nextChoice = choices[0]
+            valueDescription = str(value)
+            useNext = False
+            for c in choices:
+              (cVal, cName) = (c[0],c[1])
+              if(useNext):
+                nextChoice = c
+                useNext = False
+              if(str(value) == str(cVal)):
+                valueDescription = cName
+                useNext = True
+                if len(c) == 3:
+                  cAction = c[2]
+
+            # What should happen if this option is clicked -
+            # set the associated option to the next value in sequence
+            onClick = "set:%s:%s" % (variable, str(nextChoice[0]))
+            if cAction:
+              onClick += "|%s" % cAction
+            onClick += "|options:save"
+            onClick += "|set:needRedraw:1"
 
           y = y4 + (row) * dy
           w = w1 - (x4-x1)
